from pymimir import StateSpace

from pathlib import Path

ROOT_DIR = (Path(__file__).parent.parent.parent.parent).absolute()


def test_state_space():
    """ Test the construction of state space
    """
    domain_filepath = str(ROOT_DIR / "data" / "gripper" / "domain.pddl")
    problem_filepath = str(ROOT_DIR / "data" / "gripper" / "p-2-0.pddl")

    state_space = StateSpace.create(domain_filepath, problem_filepath)

    assert state_space.get_num_states() == 28
    assert state_space.get_num_transitions() == 104
    assert state_space.get_num_goal_states() == 2
    assert state_space.get_num_deadend_states() == 0


def test_state_space_parallel():
    """ Test the parallel construction of state space
    """
    domain_filepath = str(ROOT_DIR / "data" / "gripper" / "domain.pddl")
    problem_filepath_1 = str(ROOT_DIR / "data" / "gripper" / "p-1-0.pddl")
    problem_filepath_2 = str(ROOT_DIR / "data" / "gripper" / "p-2-0.pddl")
    problem_filepaths = [problem_filepath_1, problem_filepath_2]

    state_spaces = StateSpace.create(domain_filepath, problem_filepaths)

    assert len(state_spaces) == 2
    assert state_spaces[0].get_num_states() == 8
    assert state_spaces[1].get_num_states() == 28



# Import all classes for better IDE support

# Formalism
from _pymimir import (
    Action,
    AssignOperatorEnum,
    Axiom,
    BinaryOperatorEnum,
    ConditionalEffect,
    DerivedAtom,
    DerivedGroundAtom,
    DerivedGroundLiteral,
    DerivedLiteral,
    DerivedPredicate,
    Domain,
    FluentAtom,
    FluentGroundAtom,
    FluentGroundLiteral,
    FluentLiteral,
    FluentPredicate,
    Function,
    FunctionExpression,
    FunctionExpressionBinaryOperator,
    FunctionExpressionFunction,
    FunctionExpressionMinus,
    FunctionExpressionMultiOperator,
    FunctionExpressionNumber,
    FunctionSkeleton,
    GroundFunction,
    GroundFunctionExpression,
    GroundFunctionExpressionBinaryOperator,
    GroundFunctionExpressionFunction,
    GroundFunctionExpressionMinus,
    GroundFunctionExpressionMultiOperator,
    GroundFunctionExpressionNumber,
    MultiOperatorEnum,
    NumericFluent,
    Object,
    OptimizationMetric,
    OptimizationMetricEnum,
    PDDLFactories,
    PDDLParser,
    Problem,
    Requirements,
    RequirementEnum,
    SimpleEffect,
    StaticAtom,
    StaticGroundAtom,
    StaticGroundLiteral,
    StaticLiteral,
    StaticPredicate,
    Term,
    TermObject,
    TermVariable,
    UniversalEffect,
    Variable
)

# Search
from _pymimir import (
    AStarAlgorithm,
    BlindHeuristic,
    BrFSAlgorithm,
    DebugBrFSAlgorithmEventHandler,
    DebugGroundedAAGEventHandler,
    DebugLiftedAAGEventHandler,
    DefaultBrFSAlgorithmEventHandler,
    DefaultGroundedAAGEventHandler,
    DefaultIWAlgorithmEventHandler,
    DefaultLiftedAAGEventHandler,
    DefaultSIWAlgorithmEventHandler,
    FluentAndDerivedMapper,
    GroundAction,
    GroundedAAG,
    IAAG,
    IAlgorithm,
    IBrFSAlgorithmEventHandler,
    IIWAlgorithmEventHandler,
    IGroundedAAGEventHandler,
    IHeuristic,
    ILiftedAAGEventHandler,
    ISIWAlgorithmEventHandler,
    IWAlgorithm,
    LiftedAAG,
    SearchNodeStatus,
    SearchStatus,
    SIWAlgorithm,
    SuccessorStateGenerator,
    State,
    TupleIndexMapper
)

# Dataset
from _pymimir import (
    Abstraction,
    FaithfulAbstractState,
    FaithfulAbstraction,
    GlobalFaithfulAbstractState,
    GlobalFaithfulAbstraction,
    Transition,
    StateSpace
)

# Graphs
from _pymimir import (
    Certificate,
    Digraph,
    NautyGraph,
    ObjectGraph,
    ObjectGraphFactory,
    ProblemColorFunction,
    TupleGraphVertex,
    TupleGraph,
    TupleGraphFactory
)



from pymimir import FaithfulAbstraction

from pathlib import Path

ROOT_DIR = (Path(__file__).parent.parent.parent.parent).absolute()


def test_faithful_abstraction():
    """ Test the construction of faithful abstraction
    """
    domain_filepath = str(ROOT_DIR / "data" / "gripper" / "domain.pddl")
    problem_filepath = str(ROOT_DIR / "data" / "gripper" / "p-2-0.pddl")

    abstraction = FaithfulAbstraction.create(domain_filepath, problem_filepath)

    assert abstraction.get_num_states() == 12
    assert abstraction.get_num_transitions() == 46
    assert abstraction.get_num_goal_states() == 2
    assert abstraction.get_num_deadend_states() == 0


def test_faithful_abstraction_parallel():
    """ Test the parallel construction of parallel faithful abstraction
    """
    domain_filepath = str(ROOT_DIR / "data" / "gripper" / "domain.pddl")
    problem_filepath_1 = str(ROOT_DIR / "data" / "gripper" / "p-1-0.pddl")
    problem_filepath_2 = str(ROOT_DIR / "data" / "gripper" / "p-2-0.pddl")
    problem_filepaths = [problem_filepath_1, problem_filepath_2]

    abstractions = FaithfulAbstraction.create(domain_filepath, problem_filepaths)

    assert len(abstractions) == 2

    # Reduced from 8 to 6 abstract states.
    assert abstractions[0].get_num_states() == 6
    # Reduced from 28 to 12 abstract states.
    assert abstractions[1].get_num_states() == 12



#! /usr/bin/env python

from pymimir import PDDLParser, LiftedAAG, SuccessorStateGenerator

from pathlib import Path

ROOT_DIR = (Path(__file__).parent.parent.parent).absolute()

if __name__ == "__main__":
    domain_filepath = str(ROOT_DIR / "data" / "gripper" / "domain.pddl")
    problem_filepath = str(ROOT_DIR / "data" / "gripper" / "test_problem.pddl")

    # Ownership semantics rule 1: do not unintentionally free memory of PDDLParser
    # You should assign it to a variable and do not reassign it.
    pddl_parser = PDDLParser(domain_filepath, problem_filepath)
    # Ownership semantics rule 2: do not unintentionally free memory of ApplicableActionGenerator (AAG)
    # You should assign it to a variable and do not reassign it.
    lifted_aag = LiftedAAG(pddl_parser.get_problem(), pddl_parser.get_factories())
    # Ownership semantics rule 3: do not unintentionally free memory of SuccessorStateGenerator (SuccessorStateGenerator)
    # You should assign it to a variable and do not reassign it.
    ssg = SuccessorStateGenerator(lifted_aag)

    # Do some work and access data from PDDLParser, AAG, and SuccessorStateGenerator
    initial_state = ssg.get_or_create_initial_state()
    actions = lifted_aag.compute_applicable_actions(initial_state)

    print(initial_state.to_string(pddl_parser.get_problem(), pddl_parser.get_factories()))

    for action in actions:
        print(action.to_string(pddl_parser.get_factories()))
